@@ -11,6 +11,19 @@
 #' @param newdata A data frame of test predictors.
 #' @param feature A string denoting the dimension for computing lp distances.
 #' @param p A positive real number determining the norm p-norm used.
+#' @param scale A boolean indicating whether or not we want to center + scale
+#'   the features (based on the mean and sd of the training data) before calculating
+#'   the L_p norm. This is useful for computing the detachment index, but can be
+#'   less useful when we need to interpret the L_p distances.
+#' @param aggregation The aggregation used when the weightMatrix is calculated.
+#'   This can be useful for calculating the lp distances on observations in
+#'   the training data. This must be one of `average`, `oob`, or `doubleOOB`.
+#'   When newdata has fewer rows than the training data, one must also pass
+#'   the vector of training indices corresponding to the indices of the observations
+#'   in the original data set. Default is `average`.
+#' @param trainingIdx This is an optional parameter that must be set when
+#'   aggregation is set to `oob` or `doubleOOB` and the newdata is not the same
+#'   size as the training data.
 #' @return A vector of the lp distances.
 #' @examples
 #'
@@ -32,7 +45,13 @@
 #'                           feature = "Petal.Length",
 #'                           p = 2)
 #' @export
-compute_lp <- function(object, newdata, feature, p){
+compute_lp <- function(object,
+                       newdata,
+                       feature,
+                       p,
+                       scale=FALSE,
+                       aggregation="average",
+                       trainingIdx=NULL){
 
   # Checks and parsing:
   if (!inherits(object, "forestry")) {
@@ -45,10 +64,21 @@ compute_lp <- function(object, newdata, feature, p){
     stop("The submitted feature is not in the set of possible features")
   }
 
+  if (!(aggregation %in% c("average", "oob","doubleOOB"))) {
+    stop("Aggregation must be average, oob, or doubleOOB")
+  }
+
   # Compute distances
-  y_weights <- predict(object = object,
-                       newdata = newdata,
-                       weightMatrix = TRUE)$weightMatrix
+
+  # Calculate the weight matrix with the correct aggregation
+  args.predict <- list("aggregation" = aggregation,
+                       "object" = object,
+                       "newdata" = newdata,
+                       "weightMatrix" = TRUE)
+  if (!is.null(trainingIdx)) {
+    args.predict$trainingIdx <- trainingIdx
+  }
+  y_weights <- do.call(predict,args.predict)$weightMatrix
 
   if (is.factor(newdata[1, feature])) {
     # Get categorical feature mapping
@@ -71,21 +101,34 @@ compute_lp <- function(object, newdata, feature, p){
     diff_mat <- matrix(newdata[,feature],
                        nrow = nrow(newdata),
                        ncol = nrow(train_set),
-                       byrow = TRUE) !=
+                       byrow = FALSE) !=
                 matrix(train_set[,feature],
                        nrow = nrow(newdata),
                        ncol = nrow(train_set),
-                       byrow = FALSE)
+                       byrow = TRUE)
     diff_mat[diff_mat] <- 1
   } else {
-    diff_mat <- matrix(newdata[,feature],
+
+    # If we scale the features, use the mean and SD from the training data
+    if (scale) {
+      feat_mean <- mean(train_set[,feature], na.rm = TRUE)
+      feat_sd <- sd(train_set[,feature], na.rm = TRUE)
+      feature_in_newdata <- scale(newdata[,feature], center = feat_mean, scale = feat_sd)[,]
+      feature_in_traindata <- scale(train_set[,feature], center = feat_mean, scale = feat_sd)[,]
+    } else {
+      feature_in_newdata <- newdata[,feature]
+      feature_in_traindata <- train_set[,feature]
+    }
+
+
+    diff_mat <- matrix(feature_in_newdata,
                        nrow = nrow(newdata),
                        ncol = nrow(train_set),
-                       byrow = TRUE) -
-                matrix(train_set[,feature],
+                       byrow = FALSE) -
+                matrix(feature_in_traindata,
                        nrow = nrow(newdata),
                        ncol = nrow(train_set),
-                       byrow = FALSE)
+                       byrow = TRUE)
   }
 
   # Raise absoulte differences to the pth power
