@@ -39,7 +39,8 @@ public:
     bool verbose,
     bool splitMiddle,
     size_t maxObs,
-    size_t minTreesPerGroup,
+    size_t minTreesPerFold,
+    size_t foldSize,
     bool hasNas,
     bool linear,
     bool symmetric,
@@ -78,12 +79,9 @@ public:
       std::unique_ptr< std::vector< std::vector<double> >  > & split_vals,
       std::unique_ptr< std::vector< std::vector<int> >  > & naLeftCounts,
       std::unique_ptr< std::vector< std::vector<int> >  > & naRightCounts,
-      std::unique_ptr< std::vector< std::vector<size_t> >  > & leafAveidxs,
-      std::unique_ptr< std::vector< std::vector<size_t> >  > & leafSplidxs,
-      std::unique_ptr< std::vector< std::vector<size_t> >  > &
-        averagingSampleIndex,
-      std::unique_ptr< std::vector< std::vector<size_t> >  > &
-        splittingSampleIndex);
+      std::unique_ptr< std::vector< std::vector<size_t> >  > & averagingSampleIndex,
+      std::unique_ptr< std::vector< std::vector<size_t> >  > & splittingSampleIndex,
+      std::unique_ptr< std::vector< std::vector<double> >  > & weights);
 
   size_t getTotalNodeCount();
 
@@ -209,8 +207,12 @@ public:
     return _maxObs;
   }
 
-  size_t getMinTreesPerGroup() {
-    return _minTreesPerGroup;
+  size_t getminTreesPerFold() {
+    return _minTreesPerFold;
+  }
+
+  size_t getFoldSize() {
+      return _foldSize;
   }
 
   bool gethasNas() {
@@ -258,7 +260,8 @@ private:
   std::unique_ptr< std::vector<double> > _variableImportance;
   bool _splitMiddle;
   size_t _maxObs;
-  size_t _minTreesPerGroup;
+  size_t _minTreesPerFold;
+  size_t _foldSize;
   bool _hasNas;
   bool _linear;
   bool _symmetric;
