@@ -1,10 +1,13 @@
 #ifndef FORESTRYCPP_UTILS_H
 #define FORESTRYCPP_UTILS_H
 
-#include <iostream>
+#include "DataFrame.h"
 #include <vector>
 #include <string>
 #include <iostream>
+#include <random>
+#include <algorithm>
+#include <cmath>
 
 void print_vector(
   std::vector<size_t> v
@@ -26,12 +29,6 @@ double square(
     double x
 );
 
-void group_out_sample(
-    size_t groupIdx,
-    std::vector<size_t>& groupMemberships,
-    std::vector<size_t>& outputIdx,
-    std::mt19937_64& random_number_generator
-);
 
 size_t bin_to_idx(
     std::vector<bool> binary
@@ -53,10 +50,13 @@ struct tree_info {
   // observations in a leaf for a leaf node
   std::vector< long double > split_val;
   // contains the split values for regular nodes
-  std::vector< int > leafAveidx;
-  // contains the indices of observations in a leaf.
-  std::vector< int > leafSplidx;
-  // contains the indices of observations in a leaf.
+  std::vector< double > values;
+  // contains the weights used for prediction in each node
+  // 0.0 for interior nodes
+  std::vector< int > num_spl_samples;
+  // Counts of splitting samples at each node
+  std::vector< int > num_avg_samples;
+  // Contains the counts of averaging samples at each node
   std::vector< int > averagingSampleIndex;
   // contains the indices of the average set.
   std::vector< int > splittingSampleIndex;
